<?php
/* vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4: */

/** * PHP Client for KakakuCom API
 *
 * PHP versions 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 *
 * @category   Services
 * @package    Services_KakakuCom
 * @author     Shin Ohno <ganchiku@gmail.com>
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    0.4.0- v.1.0-dev (not v.1 but v1.0)
 * @see        http://apiblog.kakaku.com/
 * @since      0.1
 */

/**
 * Services_KakakuCom exception class
 */
require_once ('Services/KakakuCom/Exception.php');

/**
 * Services_KakakuCom Parameter class
 */
require_once ('Services/KakakuCom/Parameter.php');

/**
 * Services_KakakuCom
 *
 * @package Services_KakakuCom
 * @version 0.4.0
 * @author  Shin Ohno <ganchiku@gmail.com>
 */
class Services_KakakuCom
{
    // {{{ class const
    /**
     * Version of this package
     */
    const VERSION = '0.4.0';

    /**
     * URL for API
     */
	var $baseUrl = "http://api.kakaku.com/WebAPI/ItemSearch/Ver1.0/";
    #const URL = 'http://api.kakaku.com/Ver1/'; //removed

    /**
     * @var string
     */
    var $apiVersion = '1.0';

    // }}}

    // {{{ instance fields
    /**
     * useCache
     *
     * @var boolean
     * @access protected
     */
    protected $useCache = false;
    /**
     * cacheOptions
     *
     * @var array
     * @access protected
     */
    protected $cacheOptions = array();

    /**
     * responseFormat
     *
     * @var string
     * @access protected
     */
    protected $responseFormat = 'object';
    // }}}
    /**
     * Constructor
     *
     * @param boolean $useCache
     * @param array $cacheOptions
     * @access public
     * @return void
     */
    public function __construct($options = array())
    {
        $availableOptions = array('useCache', 'cacheOptions', 'responseFormat');
        foreach ($options as $key => $value) {
            if (in_array($key, $availableOptions)) {
                $this->$key = $value;
            }
        }
    }

    /**
     * Choose which Response Fomat to use(object or array)
     *
     * @param string $responseFormat
     * @access public
     * @return void
     * @throws Services_KakakuCom_InvalidParameterValue_Exception
     */
    public function setResponseFormat($responseFormat)
    {
        if ($responseFormat == 'object' or $responseFormat == 'array') {
            $this->responseFormat = $responseFormat;
        } else {
            throw new Services_KakakuCom_InvalidParameterValue_Exception;
        }
    }

    /**
     * Choose if this uses Cache_Lite.
     * If this uses Cache_Lite, then set the cacheOptions for Cache_Lite
     *
     * @param mixed $useCache
     * @param array $cacheOptions
     * @access public
     * @return void
     */
    public function setUseCache($useCache = false, $cacheOptions = array())
    {
        $this->useCache = $useCache;
        if ($useCache) {
            $this->cacheOptions = $cacheOptions;
        }
    }

    /**
     * search
     *
     * @param array $options
     * @access public
     * @throws Services_KakakuCom_InvalidParameterValue_Exception
     * @return array or simplexmlelement
     */
    public function search($options = array())
    {
		//V1.0
        $availables = array('ApiKey', 'Keyword', 'ResultSet', 'CategoryGroup', 'SortOrder', 'HitNum', 'PageNum', 'MinPrice', 'MaxPrice');//fix
        foreach (array_keys($options) as $key) {
            if (!in_array($key, $availables)) {
                throw new Services_KakakuCom_InvalidParameterValue_Exception; //fix
            }
        }
		$this->baseUrl = "http://api.kakaku.com/WebAPI/ItemSearch/Ver1.0/"; //add fix
		$this->apiVersion = '1.0';
        return $this->sendRequest('ItemSearch.aspx', $options);
    }

    /**
     * product
     *
     * @param array $options
     * @access public
     * @throws Services_KakakuCom_InvalidParameterValue_Exception
     * @return array or simplexmlelement
     */
    public function product($options = array())
    {
        $availables = array('ProductID', 'ResultSet');
        foreach (array_keys($options) as $key) {
            if (!in_array($key, $availables)) {
                throw new Services_KakakuCom_InvalidParameterValue_Exceptoin; //fix
            }
        }
		$this->baseUrl = "http://api.kakaku.com/WebAPI/ItemSearch/Ver1.0/"; //add fix
		$this->apiVersion = '1.0';
        return $this->sendRequest('ItemInfo.ashx', $options);
    }

    /**
     * provider
     *
     * @param array $options
     * @access public
     * @throws Services_KakakuCom_InvalidParameterValue_Exception
     * @return array or simplexmlelement
     */
    public function provider($options = array())
    {
        $availables = array('ZipCode', 'TypeOfLine', 'TypeOfResidence', 'TelSubscription');
        foreach (array_keys($options) as $key) {
            if (!in_array($key, $availables)) {
                throw new Services_KakakuCom_InvalidParameterValue_Exception; //fix
            }
        }
		$this->baseUrl = "http://api.kakaku.com/WebAPI/ItemSearch/Ver1.0/"; //add fix
		$this->apiVersion = '1.0';
        return $this->sendRequest('ProviderSearch.aspx', $options);
    }

    /**
     * search bbs
     *
     * @param array $options
     * @access public
     * @throws Services_KakakuCom_InvalidParameterValue_Exception
     * @return array or simplexmlelement
     */
    public function bbs($options = array())
    {
        $availables = array('ProductID', 'SortOrder', 'Filter');
        foreach (array_keys($options) as $key) {
            if (!in_array($key, $availables)) {
                throw new Services_KakakuCom_InvalidParameterValue_Exception; //fix
            }
        }
		$this->baseUrl = "http://api.kakaku.com/WebAPI/ItemSearch/Ver1.0/"; //add fix
		$this->apiVersion = '1.0';
        return $this->sendRequest('BBSInfo.asp', $options);
    }


    /**
     * sendRequest
     *
     * @param string $path
     * @param array $parameters
     * @access protected
     * @throws Services_KakakuCom_InternalServerError_Exception
     * @return array or simplexmlelement
     */
    protected function sendRequest($path, $options)
    {
        $parameters = $this->getParameters($options);

        $cacheID = '';
        // cache
        if ($this->useCache) {
            @include_once 'Cache/Lite.php';
            if (!class_exists('Cache_Lite')) {
                throw new Services_KakakuCom_InternalServerError_Exception;
            }
            $cache = new Cache_Lite($this->cacheOptions);

            $cacheID = md5($path . serialize($options));
            if ($response = $cache->get($cacheID)) {
                return $this->parseResponse($response);
            }
        }

        // request
        if (!function_exists('curl_init')) {
            throw new Services_KakakuCom_InternalServerError_Exception;
        }
        if (!$ch = curl_init()) {
            throw new Services_KakakuCom_InternalServerError_Exception;
        }
        $url = $this->baseUrl . $path; //fix
        foreach ($parameters as $key => $val) {
            $query[] = $key . '=' . urlencode(mb_convert_encoding($val, 'UTF-8', mb_detect_encoding($val)));
        }
        $url .= '?' . implode('&', $query);
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_POST, false);//fix
#        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_USERAGENT, 'User-Agent: Service_KakakuCom: ' . self::VERSION);
        $response = curl_exec($ch);
        $response = mb_convert_encoding($response, 'UTF-8', mb_detect_encoding($response));

        if (($errno = curl_errno($ch)) != 0) {
            throw new Services_KakakuCom_InternalServerError_Exception;
        }
        curl_close($ch);

        // cache
        if ($this->useCache and isset($cache)) {
            if (!$cache->save($response, $cacheID)) {
                throw new Services_KakakuCom_InternalServerError_Exception;
            }
        }
        return $this->parseResponse($response);
    }

    /**
     * parseResponse
     *
     * @param string $response
     * @access private
     * @return SimpleXMLElement or array of the response data.
     * @throws Services_KakakuCom_InternalServerError_Exception
     * @throws Services_KakakuCom_ItemNotFound_Exception
     * @throws Services_KakakuCom_TooManyItemsRequested_Exception
     * @throws Services_KakakuCom_InvalidParameterValue_Exception
     * @throws Services_KakakuCom_PlanNotFound_Exception
     */
    private function parseResponse($response)
    {
        if (!$data = simplexml_load_string($response)) {
            throw new Services_KakakuCom_InternalServerError_Exception;
        }
        // Check if the response is error.
        if (isset($data->Message)) {
            $message = trim((string)$data->Message);
            $classname = 'Services_KakakuCom_' . $message . '_Exception';
            if (!class_exists($classname)) {
                throw new Services_KakakuCom_InternalServerError_Exception;
            }
            throw new $classname;
        }

        if ($this->responseFormat == 'array') {
            $data = $this->forArray($data);
        }
        return $data;
    }

    /**
     * Parse all SimpleXMLElement to array
     *
     * @param mixed $object SimpleXMLElement or array
     * @access private
     * @return array
     */
    private function forArray($object)
    {
        $return = array();

        if (is_array($object)) {
            foreach ($object as $key => $value) {
                $return[$key] = $this->forArray($value);
            }
        } else {
            $vars = get_object_vars($object);
            if (is_array($vars)) {
                foreach ($vars as $key => $value) {
                    $return[$key] = ($key && !$value) ? null : $this->forArray($value);
                }
            } else {
                return $object;
            }
        }
        return $return;
    }

    /**
     * getParameters
     *
     * @param array $options
     * @access private
     * @throws Services_KakakuCom_InvalidParameterValue_Exception
     * @return array
     */
    private function getParameters($options)
    {
        $parameters = array();
        foreach ($options as $key => $value) {
            $classname = 'Services_KakakuCom_Parameter_' . $key;
            if (!class_exists($classname)) {
                throw new Services_KakakuCom_InvalidParameterValue_Exception;
            }
            $parameter = new $classname($value);
            $parameters[$key] = $parameter->getValue();
        }
        return $parameters;
    }
}

?>
