<?php
/* vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4: */

/**
 * Exception Class for Services_KakakuCom
 *
 * PHP versions 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 *
 * @category   Services
 * @package    Services_KakakuCom
 * @author     Shin Ohno <ganchiku@gmail.com>
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    0.4.0
 * @see        http://apiblog.kakaku.com/
 * @since      0.1
 */

/**
 * Use PEAR_Exception
 */
require_once 'PEAR/Exception.php';

abstract class Services_KakakuCom_Exception extends PEAR_Exception
{
    public function __construct()
    {
        parent::__construct($this->getErrorCode());
    }
    public abstract function getErrorMessage();
    public abstract function getErrorCode();
}

class Services_KakakuCom_ItemNotFound_Exception extends Services_KakakuCom_Exception
{
    public function getErrorMessage()
    {
        return '該当する商品がひとつもありませんでした。';
    }
    public function getErrorCode()
    {
        return 'ItemNotFound';
    }
}

class Services_KakakuCom_TooManyItemsRequested_Exception extends Services_KakakuCom_Exception
{
    public function getErrorMessage()
    {
        return '制限値を超えたアイテム数のリクエストがありました。';
    }
    public function getErrorCode()
    {
        return 'TooManyItemsRequested';
    }
}

class Services_KakakuCom_InvalidParameterValue_Exception extends Services_KakakuCom_Exception
{
    public function getErrorMessage()
    {
        return 'パラメータの値が入っていないか、不正です。';
    }
    public function getErrorCode()
    {
        return 'InvalidParameterValue';
    }
}

class Services_KakakuCom_InternalServerError_Exception extends Services_KakakuCom_Exception
{

    public function getErrorMessage()
    {
        return 'サーバは、処理を完了できませんでした。';
    }
    public function getErrorCode()
    {
        return 'InternalServerError';
    }
}

class Services_KakakuCom_PlanNotFound_Exception extends Services_KakakuCom_Exception
{
    public function getErrorMessage()
    {
        return '該当するプランがひとつもありませんでした。';
    }
    public function getErrorCode()
    {
        return 'PlanNotFound';
    }

}

class Services_KakakuCom_NoEntry_Exception extends Services_KakakuCom_Exception
{
    public function getErrorMessage()
    {
        return '該当する書込みが1件もありませんでした。';
    }
    public function getErrorCode()
    {
        return 'NoEntry';
    }
}

?>
