<?php
/* vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4: */

/**
 * PHP Client for KakakuCom API
 *
 * PHP versions 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 *
 * @category   Services
 * @package    Services_KakakuCom
 * @author     Shin Ohno <ganchiku@gmail.com>
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    0.4.0- v.1.0-dev (not v.1 but v1.0)
 * @see        http://apiblog.kakaku.com/
 * @since      0.1
 */

abstract class Services_KakakuCom_Parameter
{
    protected $value;
    public function getValue()
    {
        return $this->value;
    }
}
// Search Article
class Services_KakakuCom_Parameter_ApiKey extends Services_KakakuCom_Parameter
{
    public function __construct($value)
    {
        if (!preg_match('/^[a-zA-Z0-9]+$/', $value)) {
            throw new Services_KakakuCom_InvalidParameterValue_Exception;
        }
        $this->value = $value;
    }
}
// Search Article
class Services_KakakuCom_Parameter_Keyword extends Services_KakakuCom_Parameter
{
    public function __construct($value)
    {
        if (is_array($value)) {
            $this->value = implode(" ", $value);
        } else {
            $this->value = $value;
        }
    }
}
// Search Article
class Services_KakakuCom_Parameter_PageNum extends Services_KakakuCom_Parameter
{
    public function __construct($value)
    {
        if (!preg_match('/^[0-9]+$/', $value)) {
            throw new Services_KakakuCom_InvalidParameterValue_Exception;
        }
        $this->value = $value;
    }
}
// Fetch Item
class Services_KakakuCom_Parameter_ProductID extends Services_KakakuCom_Parameter
{
    const MAX = 5;
    public function __construct($value)
    {
        if (is_array($value) and count($value) >= self::MAX) {
            $this->value = implode(",", $value);
        } else {
            $this->value = $value;
        }
    }
}

abstract class Services_KakakuCom_Available_Parameter extends Services_KakakuCom_Parameter
{
    protected $availableValues = array();
    public function __construct($value)
    {
        if (!is_string($value) and !in_array($value, $this->availableValues)) {
            throw new Services_KakakuCom_InvalidParameterValue_Exception;
        }
        $this->value = $value;
    }
}
// Search Article
// Fetch Item
class Services_KakakuCom_Parameter_ResultSet extends Services_KakakuCom_Available_Parameter
{
    function __construct($value)
    {
        $this->availableValues = array('mini', 'medium');
        parent::__construct($value);
    }
}

// Search Article
class Services_KakakuCom_Parameter_CategoryGroup extends Services_KakakuCom_Available_Parameter
{
    function __construct($value)
    {
		//v.1.0(new)
        $this->availableValues = array('Pc', 'Kaden', 'Camera', 'Game', 'Gakki', 'Kuruma', 'Sports', 'Brand', 'Baby', 'Pet', 'Beauty_Health', 'ALL');
        parent::__construct($value);
    }
}

// Search Article
class Services_KakakuCom_Parameter_SortOrder extends Services_KakakuCom_Available_Parameter
{
    function __construct($value)
    {
		//v.1.0
        $this->availableValues = array('pricerank', '-pricerank', 'daterank', 'popularityrank');
        parent::__construct($value);
    }
}

// Search Article
class Services_KakakuCom_Parameter_HitNum extends Services_KakakuCom_Available_Parameter
{
    function __construct($value)
    {
        if (!preg_match('/^[0-9]+$/', $value)) {
            throw new Services_KakakuCom_InvalidParameterValue_Exception;
        }
        $this->value = $value;
    }
}

// Search Article
class Services_KakakuCom_Parameter_MinPrice extends Services_KakakuCom_Available_Parameter
{
    function __construct($value)
    {
        if (!preg_match('/^[0-9]+$/', $value)) {
            throw new Services_KakakuCom_InvalidParameterValue_Exception;
        }
        $this->value = $value;
    }
}

// Search Article
class Services_KakakuCom_Parameter_MaxPrice extends Services_KakakuCom_Available_Parameter
{
    function __construct($value)
    {
        if (!preg_match('/^[0-9]+$/', $value)) {
            throw new Services_KakakuCom_InvalidParameterValue_Exception;
        }
        $this->value = $value;
    }
}

// Search Provider
class Services_KakakuCom_Parameter_ZipCode extends Services_KakakuCom_Parameter
{
    function __construct($value)
    {
        if (!preg_match('/^[0-9]{3}-[0-9]{4}$/', $value)) {
            throw new Services_KakakuCom_InvalidParameterValue_Exception;
        }
        $this->value = $value;
    }
}

// Search Provider
class Services_KakakuCom_Parameter_TypeOfLine extends Services_KakakuCom_Available_Parameter
{
    function __construct($value)
    {
        $this->availableValues = array('ADSL', 'FTTH', 'ALL');
        parent::__construct($value);
    }
}
// Search Provider
class Services_KakakuCom_Parameter_TypeOfResidence extends Services_KakakuCom_Available_Parameter
{
    function __construct($value)
    {
        $this->availableValues = array('House', 'Mansion', 'ALL');
        parent::__construct($value);
    }
}
// Search Provider
class Services_KakakuCom_Parameter_TelSubscription extends Services_KakakuCom_Available_Parameter
{
    function __construct($value)
    {
        $this->availableValues = array('Enable', 'Disable', 'ALL');
        parent::__construct($value);
    }
}
// Search BBS
class Services_KakakuCom_Parameter_Filter extends Services_KakakuCom_Available_Parameter
{
    function __construct($value)
    {
        $this->availableValues = array('1', '2', '3');
        parent::__construct($value);
    }
}

?>
